-- Model: CA - Fire spread in a forest (numeric attributes)
--
-- Author: Pedro Andrade & Tiago Carneiro
--
-- Lesson: TerraME models MUST have the following structure
--             1) Define space
--				2) Define behavior
--				3) Define time
--				4) Run simulations 
--

--------------------------------------------------------------
-- GLOBALS
--------------------------------------------------------------

FOREST = 1
BURNING = 2
BURNED = 3

--------------------------------------------------------------
-- MODEL (1. space, 2. behavior, and 3. time)
--------------------------------------------------------------
-------------
--# SPACE #
-------------

world = CellularSpace{
	xdim = 50,
	ydim = 50
}

world:createNeighborhood{
	strategy = "vonneumann",
	self = false
}

forEachCell(world, function(cell)
	cell.cover = FOREST
end)

----------------
--# BEHAVIOR #
----------------
world:sample().cover = BURNING

update = function(cs)
	forEachCell(cs, function(cell)
		if cell.past.cover == FOREST then
			forEachNeighbor(cell, function(cell, neighbor)
				if neighbor.past.cover == BURNING then
					cell.cover = BURNING
				end
			end)
		elseif cell.past.cover == BURNING then
			cell.cover = BURNED
		end
	end)
end

-----------
--# TIME #
-----------

t = Timer{
	Event{action = function()
		world:synchronize()
		update(world)
		world:notify()
	end}
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

legend = Legend {
	grouping = "uniquevalue",
	colorBar = {
		{value = FOREST, color = "green"},
		{value = BURNING, color = "red"},
		{value = BURNED, color = "brown"},
	}
}

Observer{
	subject = world,
	attributes = {"cover"},
	legends = {legend}
}


world:notify()  -- needed just if you want to see the initial state (before simulation runs)

t:execute(90)

