-- Model: CA - Fire spread in a forest (striing attribute)
--
-- Author: Pedro Andrade & Tiago Carneiro
--
-- Lesson: TerraME models MUST have the following structure
--             1) Define space
--				2) Define behavior
--				3) Define time
--				4) Run simulations 


--------------------------------------------------------------
-- MODEL (1. space, 2. behavior, and 3. time)
--------------------------------------------------------------

-------------
--# SPACE #
-------------

world = CellularSpace{
	xdim = 50,
	ydim = 50
}

world:createNeighborhood{
	strategy = "vonneumann",
	self = false
}

forEachCell(world, function(cell)
	cell.cover = "forest"
end)

----------------
--# BEHAVIOR #
----------------

world:sample().cover = "burning"


update = function(cs)
	forEachCell(cs, function(cell)
		if cell.past.cover == "forest" then
			forEachNeighbor(cell, function(cell, neighbor)
				if neighbor.past.cover == "burning" then
					cell.cover = "burning"
				end
			end)
		elseif cell.past.cover == "burning" then
			cell.cover = "burned"
		end
	end)
end



-----------
--# TIME #
-----------

t = Timer{
	Event{action = function()
		world:synchronize()
		update(world)
		world:notify()
	end}
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

legend = Legend {
	grouping = "uniquevalue",
	type = "string",
	colorBar = {
		{value = "forest", color = "green"},
		{value = "burning", color = "red"},
		{value = "burned", color = "brown"},
	}
}

Observer{
	subject = world,
	attributes = {"cover"},
	legends = {legend}
}

world:notify()  -- needed just if you want to see the initial state (before simulation runs)

t:execute(90)

