-- Model: CA - Fire spread in a forest (4 numeric states)
--					STOCHASTIC INITIAL CONDITON: empty space
--
-- Author: Pedro Andrade & Tiago Carneiro
--
-- Lesson: TerraME models MUST have the following structure
--             1) Define space
--				2) Define behavior
--				3) Define time
--				4) Run simulations 
--

--------------------------------------------------------------
-- GLOBALS
--------------------------------------------------------------

FOREST = 1
BURNING = 2
BURNED = 3
EMPTY = 4

rand = Random()

--------------------------------------------------------------
-- MODEL (1. space, 2. behavior, and 3. time)
--------------------------------------------------------------

-------------
--# SPACE #
-------------

world = CellularSpace{
	xdim = 50,
	ydim = 50
}

world:createNeighborhood{
	strategy = "vonneumann",
	self = false
}

forEachCell(world, function(cell)
	if rand:number() > 0.1 then
		cell.cover = FOREST
	else
		cell.cover = EMPTY
	end
end)

----------------
--# BEHAVIOR #
----------------

world:sample().cover = BURNING

update = function(cs)
	forEachCell(cs, function(cell)
		if cell.past.cover == FOREST then
			forEachNeighbor(cell, function(cell, neighbor)
				if neighbor.past.cover == BURNING then
					cell.cover = BURNING
				end
			end)
		elseif cell.past.cover == BURNING then
			cell.cover = BURNED
		end
	end)
end

-----------
--# TIME #
-----------

t = Timer{
	Event{action = function(e)
		world:synchronize()
		update(world)
		world:notify()
	end}
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

legend = Legend {
	grouping = "uniquevalue",
	colorBar = {
		{value = FOREST, color = "green"},
		{value = BURNING, color = "red"},
		{value = BURNED, color = "brown"},
		{value = EMPTY, color = "white"}
	}
}

Observer{
	subject = world,
	attributes = {"cover"},
	legends = {legend}
}

world:notify()  -- needed just if you want to see the initial state (before simulation runs)

t:execute(100)

